package redSocialGrupos;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.SortedSet;


public class AlgoHierarchicalClustering {
	private double maxDistance =0;
	private List<Cluster> clusters = null;
	private List<Cluster> clustersAntiguos = null;
	private double maxMemory;
	private long iterationCount;
	private long numClustersCreados;
	private double umbralIdentidad; 
	

	public AlgoHierarchicalClustering(double maxDistance, double umbralIdentidad) {
		this.clusters=null;
		this.clustersAntiguos=null;
		this.maxDistance = maxDistance;
		this.umbralIdentidad=umbralIdentidad;
		this.numClustersCreados=0;
	}
	
	public List<Cluster> runAlgorithm(List<RedSocialNodo> nodos){ 
		if(clusters!=null)
			clustersAntiguos=clusters;
		
		clusters = new ArrayList<Cluster>();

		for(int i=0; i<nodos.size(); i++){
			RedSocialNodo nodo = nodos.get(i);
			numClustersCreados++;
			Cluster cluster = new Cluster(nodo.getDimensiones(),this.numClustersCreados);
			cluster.addNodo(nodo);
			cluster.setMean(nodo);
			clusters.add(cluster);
		}
		
		boolean changed = false;
		do {
			changed = mergeTheClosestCluster();
			checkMemory();
		} while (changed);

		if(clustersAntiguos!=null){
			trazabilidadClusters();
		}
		
		Collections.sort(clusters);
		
		return clusters;
	}
	
	private void trazabilidadClusters(){		
		for(int i=0; i<clustersAntiguos.size(); i++){
			Cluster ca=clustersAntiguos.get(i);
			
			Cluster clusterCercano=null;
			double distanciaMin=Double.MAX_VALUE;
			
			for(int j=0; j<clusters.size(); j++){
				Cluster cn=clusters.get(j);
				
				double distancia=ca.distanciaCluster(cn);
				
				if(distancia<=umbralIdentidad && distancia<distanciaMin){
					distanciaMin=distancia;
					clusterCercano=cn;
				}
			}
			
			if(clusterCercano!=null)
				clusterCercano.cambiarID(ca);
			
		}
		
	}
	
	
	private boolean mergeTheClosestCluster() {
		Cluster clusterToMerge1 = null;
		Cluster clusterToMerge2 = null;
		double minClusterDistance = Integer.MAX_VALUE;

		for (int i = 0; i < clusters.size(); i++) {
			for (int j = i + 1; j < clusters.size(); j++) {
				
				double distance = euclideanDistance(clusters.get(i).getmean(), clusters.get(j).getmean());
				if (distance < minClusterDistance && distance < maxDistance) {
					minClusterDistance = distance;
					clusterToMerge1 = clusters.get(i);
					clusterToMerge2 = clusters.get(j);
				}
			}
		}

		if (clusterToMerge1 == null) {
			return false;
		}

		for(RedSocialNodo nodo : clusterToMerge2.getNodos()){
			clusterToMerge1.addNodo(nodo);
		}
		clusterToMerge1.recomputeClusterMean();
		clusters.remove(clusterToMerge2);

		iterationCount++;
		return true;
	}

	private double euclideanDistance(Vector vector1, Vector vector2) {
		double sum =0;	
		for(int i=0; i< vector1.data.length; i++){
			sum += Math.pow(vector1.data[i] - vector2.data[i], 2);
		}
		return Math.sqrt(sum);
	}
	
	private void checkMemory() {
		double currentMemory = ((double) (Runtime.getRuntime()
				.totalMemory() / 1024) / 1024)
				- ((double) (Runtime.getRuntime().freeMemory() / 1024) / 1024);
		if (currentMemory > maxMemory) {
			maxMemory = currentMemory;
		}
	}
	
	public void visualizarResultados(){
		for(int i=0; i< clusters.size(); i++){
			if(clusters.get(i).getNodos().size() >= 1){
				System.out.print(clusters.get(i).toString());
				if(i < clusters.size()-1){
					System.out.println();
				}
			}
		}
	}
	
	private List<Cluster> getClustersRepresentativos(List<Cluster> lista){ 
		List<Cluster> representativos=new LinkedList<Cluster>();
		
		for(Cluster c:lista){
			if(c.getNodos().size()>(5*RedSocialModelo.getNumNodes()/100))
				representativos.add(c);
		}
		
		return representativos;
	}
	
	public List<Cluster> getClustersRepresentativos(){ 	
		List<Cluster> lista=new LinkedList<Cluster>();
		
		for(Cluster c:clusters){
			if(c.getNodos().size()>(5*RedSocialModelo.getNumNodes()/100))
				lista.add(c);
		}
		
		return lista;
	}
	
	public int getNumClustersRepresentativos(){
		return getClustersRepresentativos(clusters).size();
	}
	
	
	
	


	public double getTamMovClustersRepresentativos1(){
		double indicador=0;
		
		if(clustersAntiguos!=null){
			List<Cluster> representativosAntiguos=getClustersRepresentativos(clustersAntiguos);
			List<Cluster> representativosActuales=getClustersRepresentativos(clusters);
						
			for(int n=0; n<representativosAntiguos.size(); n++){
				Cluster cAnt=representativosAntiguos.get(n);
				double posAnt=n;
				double posAct=representativosActuales.indexOf(cAnt);
				
				if(posAct<0){
					posAct=Math.max(representativosAntiguos.size(), representativosActuales.size());
				}
				
				indicador+=Math.abs(posAnt-posAct);
			}
			
			for(int n=0; n<representativosActuales.size(); n++){
				Cluster cAct=representativosActuales.get(n);
				double posAct=n;
				double posAnt=representativosAntiguos.indexOf(cAct);
				
				if(posAnt<0){
					posAnt=Math.max(representativosAntiguos.size(), representativosActuales.size());
					indicador+=Math.abs(posAnt-posAct);
				}	
			}
		}
		
		return indicador;
	}
	
	public double getTamMovClustersRepresentativos2(){
		double indicador=0;
		
		if(clustersAntiguos!=null){
			List<Cluster> representativosAntiguos=getClustersRepresentativos(clustersAntiguos);
			List<Cluster> representativosActuales=getClustersRepresentativos(clusters);		
			
			for(int n=0; n<representativosAntiguos.size(); n++){
				Cluster cAnt=representativosAntiguos.get(n);
				double posAnt=((double)n/representativosAntiguos.size())+((double)1/representativosAntiguos.size())/2;
				double posAct=((double)representativosActuales.indexOf(cAnt)/representativosActuales.size())+((double)1/representativosActuales.size())/2;
				
				if(posAct<0){
					posAct=1+((double)1/representativosActuales.size())/2;
				}
				
				indicador+=Math.abs(posAnt-posAct);
			}
			
			for(int n=0; n<representativosActuales.size(); n++){
				Cluster cAct=representativosActuales.get(n);
				double posAct=((double)n/representativosActuales.size())+((double)1/representativosActuales.size())/2;
				double posAnt=((double)representativosAntiguos.indexOf(cAct)/representativosAntiguos.size())+((double)1/representativosAntiguos.size())/2;
				
				if(posAnt<0){
					posAnt=1+((double)1/representativosAntiguos.size())/2;
					indicador+=Math.abs(posAnt-posAct);
				}	
			}
		}

		return indicador;
	}
	
	public List<Cluster> getClusters(){ return clusters;}
}
