package redSocialGrupos;

import uchicago.src.collection.NewMatrix;
import uchicago.src.collection.RangeMap;
import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.sim.analysis.BinDataSource;
import uchicago.src.sim.analysis.DataSource;
import uchicago.src.sim.analysis.Histogram;
import uchicago.src.sim.analysis.NetSequenceGraph;
import uchicago.src.sim.analysis.OpenHistogram;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.PlotModel;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.BasicAction;
import uchicago.src.sim.engine.Controller;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.engine.SimpleModel;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.network.DefaultNode;
import uchicago.src.sim.network.NetUtilities;
import uchicago.src.sim.space.Object2DGrid;


import java.awt.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.instrument.Instrumentation;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javax.xml.datatype.Duration;




public class RedSocialModelo extends MultiExperimentoSimple{
	public static int dimensiones=2;
	
	private static int numNodes = 75;
	public static void setNumNodes(int n) {
		numNodes = n;
	}
	public static int getNumNodes() {
		return numNodes;
	}
	

	private static int spaceSize = 500;
	public static int getSpaceSize() {
	    return spaceSize;
	}
	public static void setSpaceSize(int size) {
		spaceSize = size;
	}
	
	private static double distanciaMax=spaceSize;
	public static double getDistanciaMax() {
	    return distanciaMax;
	}
	
	private static double gamma=6;
	public static double getGamma() {
	    return gamma;
	}
	public static void setGamma(double gamma) {
		RedSocialModelo.gamma = getDecimal(3, gamma);
	}
	
	private static double efectoProximidadAlfa=24;
	public static double getEfectoProximidadAlfa() {
	    return efectoProximidadAlfa;
	}
	public static void setEfectoProximidadAlfa(double efectoProximidadAlfa) {
		RedSocialModelo.efectoProximidadAlfa = getDecimal(3, efectoProximidadAlfa);
	}
	
	private static double probMutar=0.003;
	public static double getProbMutar() {
	    return probMutar;
	}
	public static void setProbMutar(double probMutar) {
		RedSocialModelo.probMutar = getDecimal(3, probMutar);
	}
	
	private List<RedSocialArista> todasPosiblesAristas;
	private AbstractGraphLayout graphLayout;
	public static int numAristas=0;
	private AlgoHierarchicalClustering algo; 
	private int duracionSimulacion=6000;
	private int numTicksConsiderados=200;
	private DecimalFormat formateador;
	private boolean primeraEjecucion=true;
	private String contenidoVirtual="";
	private BufferedWriter bw;
	private static String ficheroEntrada="parametros.xml",
	ficheroSalida="Prueba.txt";

	
	public void setup() { 
		super.setup();
		super.shuffle=true;
		super.autoStep=false;
		this.numAristas=0;
		this.todasPosiblesAristas=null;
		RedSocialModelo.distanciaMax=Math.sqrt(Math.pow(spaceSize, 2)+Math.pow(spaceSize,2));

		this.algo= new AlgoHierarchicalClustering(50, 20);
		accionesPrimeraEjecucion();
	}
	
	private void mostrarParametros(){
		System.out.println("[N="+numNodes+"],[p="+probMutar+"],[alfa="+efectoProximidadAlfa+"],[beta="+gamma+"],[t="+getTick()+"],[T="+duracionSimulacion+"],[e="+getRepeticion()+"],[dimensiones="+dimensiones+"]");
	}
	
	private void accionesPrimeraEjecucion(){
		if(primeraEjecucion){
			primeraEjecucion=false;
			
			formateador= new DecimalFormat("########.###");
			DecimalFormatSymbols dfs = formateador.getDecimalFormatSymbols();
			dfs.setDecimalSeparator(',');
			formateador.setDecimalFormatSymbols(dfs);
			
			System.out.println("Creando Cabeceras.");
			
			contenidoVirtual+=
				"t;e;p;alfa;beta;NP;" +
				"id;NG;I1;I2";
				
			for(int i=0; i<this.dimensiones; i++)
				contenidoVirtual+=";x"+(i+1);
					
			contenidoVirtual+="\n";
			
			System.out.println("Creando Archivo.");
			
			File salidaF = new File(ficheroSalida);
				
			salidaF.delete();
				
			FileWriter fw=null;
			try {
				fw = new FileWriter(salidaF, true);
				bw = new BufferedWriter(fw);
				bw.write(contenidoVirtual);		
				contenidoVirtual="";
				bw.flush();	
			}catch (IOException e) {
				e.printStackTrace();
				System.exit(-1);
			}
		}
	}

	private void liberarMemoria(){

		if(contenidoVirtual.length()>50000){
			System.out.println("Volcando por exceso: "+contenidoVirtual.length());
			volcarInformacion();
		}
		
		if(((double)Runtime.getRuntime().freeMemory()/Runtime.getRuntime().totalMemory())<0.30){
			System.out.println("Basurero: "+(double)Runtime.getRuntime().freeMemory()/Runtime.getRuntime().totalMemory());
			System.gc();
		}
	}

	private void volcarInformacion(){
		try {				
			bw.append(contenidoVirtual);
			contenidoVirtual="";
			bw.flush();
		}catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		}
	}
  
	public void begin(){
		mostrarParametros();
		buildModel();
		buildDisplay();
		buildSchedule();
	}
  
	public void buildModel() {
		for (int i = 0; i<numNodes; i++) {
			double[] coordenadas=new double[dimensiones];

			for(int c=0; c<coordenadas.length; c++){
	  	  		coordenadas[c]=Math.random()*spaceSize;
			} 
			
			RedSocialNodo btn = new RedSocialNodo(coordenadas, i);
			btn.allowResizing(false);
			btn.setWidth(10);
			btn.setHeight(10);
			btn.setColor(new Color((int)(Math.random()*255), (int)(Math.random()*255), (int)(Math.random()*255)));
	     
			agentList.add(btn);            
		}
    
		this.todasPosiblesAristas=generaTodasPosiblesAristas();
	}
	
	
	private List<RedSocialArista> generaTodasPosiblesAristas(){
		List<RedSocialArista> dev=new LinkedList();
		
		 for (int i = 0; i < agentList.size(); i++) {
			 
			 RedSocialNodo agentei = (RedSocialNodo)agentList.get(i);
			 
			 for (int j = i+1; j < agentList.size(); j++) {
				 RedSocialNodo agentej = (RedSocialNodo)agentList.get(j);
				 
				 RedSocialArista aij=new RedSocialArista(agentei, agentej);
				 
				 dev.add(aij);
			 }
		 }
		    	
		return dev;
	}

	private void buildDisplay(){
		graphLayout = new DefaultGraphLayout(agentList, spaceSize, spaceSize);
	}
  
  	public void step(){  		
  		relacionPorLiderazgo();
  		mutarAgentes();
  		clustering();
	
  		if(getTick()%200==0)
  			System.out.println(getTick());	
  		
		graphLayout.updateLayout();

  		defaultStep();  
  	}
	  
	private void relacionPorLiderazgo() {	
		  Collections.shuffle(todasPosiblesAristas);
		  
		  for (int i = 0; i < todasPosiblesAristas.size(); i++) {
		    	RedSocialArista ai = (RedSocialArista)todasPosiblesAristas.get(i);
		    	ai.getAgenteOrigen().relacionarse(ai.getAgenteDestino());	
		  }  
	}
	
	private void mutarAgentes() {		  
		  for (int i = 0; i < agentList.size(); i++) {
		    	RedSocialNodo agentei = (RedSocialNodo)agentList.get(i);
		    	
		    	if(Math.random()<this.probMutar)
		    		agentei.mutar();	
		  }  
	}
	
	private void clustering(){
		  algo.runAlgorithm(agentList);
	}

	public static double distanciaCentro(double[] coordenadas){
		double ds=0;
		  
		for(int i=0; i<coordenadas.length; i++){
  			ds+=Math.pow(coordenadas[i]-((double)spaceSize/2), 2);
  		}
  		
  		ds=Math.sqrt(ds);
		  
		return ds;
	}

	public String getName() {
			return "Modelo";
	}
	
	public static void main(String[] args) {
		uchicago.src.sim.engine.SimInit init = new uchicago.src.sim.engine.SimInit();
		RedSocialModelo model = new RedSocialModelo();

		if(args!=null && args.length>0){
			if(args.length==2){
				model.duracionSimulacion=Integer.parseInt(args[0]);
				model.numTicksConsiderados=Integer.parseInt(args[1]);
			}else
				if(args.length==4){
					model.ficheroEntrada=args[0];
					model.ficheroSalida=args[1];
					model.duracionSimulacion=Integer.parseInt(args[2]);
					model.numTicksConsiderados=Integer.parseInt(args[3]);
				}else
					if(args.length==5){
						model.ficheroEntrada=args[0];
						model.ficheroSalida=args[1];
						model.duracionSimulacion=Integer.parseInt(args[2]);
						model.numTicksConsiderados=Integer.parseInt(args[3]);
						model.dimensiones=Integer.parseInt(args[4]);
					}else{
						throw new RuntimeException("Argumentos incorrectos");
					}
		}
		
		init.loadModel(model, ficheroEntrada, true);	
	}

  
	public RedSocialModelo() {
		params = new String[] {"numNodes", "spaceSize", "gamma", "efectoProximidadAlfa", "probMutar", "repeticion", "fin"};
	}

	
	protected void recogidaDatosSeries() {

		if(getTick()>(this.duracionSimulacion-this.numTicksConsiderados)){
			List<Cluster> clusters=algo.getClusters();
			
			double I1=algo.getTamMovClustersRepresentativos1();
			double I2=algo.getTamMovClustersRepresentativos2();
			
			for(int i=0; i<clusters.size(); i++){
				Cluster ci=clusters.get(i);
			
				contenidoVirtual+=""+(int)getTick()+";"+getRepeticion()+";"+
				formateador.format(probMutar)+";"+formateador.format(efectoProximidadAlfa)+";"+formateador.format(gamma)+";"+
				numNodes+";"+ci.ID+";"+ci.getNodos().size()+";"+formateador.format(I1)+";"+formateador.format(I2);
				
				
				for(int d=0; d<this.dimensiones; d++)
					contenidoVirtual+=";"+formateador.format(ci.getmean().data[d]);
				
				contenidoVirtual+="\n";
			}
		}
		
		liberarMemoria();
	}
	
	protected void accionFinTanda() {	
		System.out.println("tanda");
		System.out.println("Volcando programado: "+contenidoVirtual.length());
		volcarInformacion();
	}

	protected void ultimaAccion() {
		try {	
			System.out.println("Cerrando archivo");
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		}
	}
	
	protected double getTick() {
		return getTickCount();
	}

	protected int getTickMax() {
		return this.duracionSimulacion;
	}

	protected void recogidaDatosHistogramas(Object agente) {
	}
	
	public static double getDecimal(int numeroDecimales, double decimal){
		decimal = decimal*(java.lang.Math.pow(10, numeroDecimales));
		decimal = java.lang.Math.round(decimal);
		decimal = decimal/java.lang.Math.pow(10, numeroDecimales);

		return decimal;  
	}
}
