

package redSocialGrupos;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;


import uchicago.src.collection.RangeMap;
import uchicago.src.sim.gui.OvalNetworkItem;
import uchicago.src.sim.gui.RectNetworkItem;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.network.DefaultDrawableNode;
import uchicago.src.sim.network.DefaultNode;
import uchicago.src.sim.network.Edge;
import uchicago.src.sim.util.Random;

public class RedSocialNodo extends DefaultDrawableNode {
	private String etiquetaCluster="";
	private int who; 
	public double[] coordenadas;
	private Cluster cluster; 

	public RedSocialNodo(double[] coordenadas, int who) {
		if(coordenadas.length>=2)
			init(coordenadas[0], coordenadas[1]);
		else
			init(coordenadas[0], 0); 
		
		this.coordenadas=coordenadas;
		this.who = who;    
		this.cluster=null;
	}

	public void init(double x, double y) {
		setDrawable(new OvalNetworkItem(x, y));
	}
	
	public void setEtiquetaCluster(String texto){
		this.etiquetaCluster=texto;
	}
	
	public void draw(SimGraphics g) {
		super.draw(g);
		
		if(this.etiquetaCluster!=null && !this.etiquetaCluster.equals("")){
			g.setFont(super.getFont());
			g.setDrawingParameters(50, 10, 0);
			g.setDrawingCoordinates((float)(this.getX()-20), (float)(this.getY()-10), 0f);
			g.drawString(this.etiquetaCluster, getColor());
		}
	}

  	public void relacionarse(RedSocialNodo agentej) {	
		if(this!=agentej && Math.random()<probabilidadEnlace(agentej)){
			makeEdgeToFrom(agentej);			
			interaccion(agentej);				
		}else{
			eliminarArista(agentej);	
		}
  	}
  	
  	private double probabilidadEnlace(RedSocialNodo agentej){
  		double p=0;
	  
  		p=Math.pow(1- (distanciaSocial(agentej)/RedSocialModelo.getDistanciaMax()), RedSocialModelo.getEfectoProximidadAlfa());
  
  		return p;
  	}
  	
	private boolean makeEdgeToFrom(RedSocialNodo node) {
		boolean seCreaAmistad=false;
	
		if (! hasEdgeTo(node)) {
			Edge edge = new RedSocialArista(this, node);
			addOutEdge(edge);
			node.addInEdge(edge);
			Edge otherEdge = new RedSocialArista(node, this);
			node.addOutEdge(otherEdge);
			addInEdge(otherEdge);
    
			RedSocialModelo.numAristas++;
			seCreaAmistad=true;
		}
  	
		return seCreaAmistad;
	}
	
  	private void interaccion(RedSocialNodo agentej){
  		double[] coordi = this.coordenadas;
  		double[] coordj=agentej.coordenadas;
	  
  		double distancia=distanciaSocial(agentej);
	 
  		if(distancia>0){
  			double[] coordiju=new double[coordj.length];
  			{
	  			for(int i=0; i<coordj.length; i++){
	  				coordiju[i]=(coordj[i]-coordi[i])/distancia;
	  			}
  			}
		  
  			double visibilidadTotal=calculaVisibilidad()+agentej.calculaVisibilidad();		  
  			double avanceij=Math.pow((agentej.calculaVisibilidad()/visibilidadTotal), RedSocialModelo.getGamma())*distancia;
  			double avanceji=Math.pow((this.calculaVisibilidad()/visibilidadTotal), RedSocialModelo.getGamma())*distancia;
			
  			{
	  			for(int i=0; i<coordi.length; i++){
	  				coordi[i]+=avanceij*coordiju[i];
	  			}
  			}
		  
  			{
	  			for(int i=0; i<coordj.length; i++){
	  				coordj[i]-=avanceji*coordiju[i];
	  			}
  			}
		  
  			this.coordenadas=coordi;
  			if(coordi.length>=2){
	  			this.setX(coordi[0]);
	  			this.setY(coordi[1]);
  			}
		  
  			agentej.coordenadas=coordj;
  			if(coordj.length>=2){
	  			agentej.setX(coordj[0]);
	  			agentej.setY(coordj[1]);
  			}
  		}
  	}
  	
    private double calculaVisibilidad(){
  	  	double v=0;
  	  
  	  	if(RedSocialModelo.numAristas>0)
  	  		v=(double)this.getOutDegree()/RedSocialModelo.numAristas;
  		  
  	  	return v;
    }
    
  	private double distanciaSocial(RedSocialNodo agentei){
  		double ds=0;
	  
  		for(int i=0; i<coordenadas.length; i++){
  			ds+=Math.pow((coordenadas[i]-agentei.coordenadas[i]), 2);
  		}
  		
  		ds=Math.sqrt(ds);
	  
  		return ds;
  	}
  	
  	private boolean eliminarArista(RedSocialNodo agentej) {
  		boolean seDestruye=false;
	  
  		if(hasEdgeToOrFrom(agentej)){
  			removeEdgesTo(agentej);
  			agentej.removeEdgesFrom(this);
  			removeEdgesFrom(agentej);
  			agentej.removeEdgesTo(this);
  			RedSocialModelo.numAristas--;
  			seDestruye=true;
  		}
	  
  		return seDestruye;
  	}
   
    public void mutar(){
  		for(int i=0; i<coordenadas.length; i++){
  			coordenadas[i]=Math.random()*RedSocialModelo.getSpaceSize();
  		}  
  	  	
  	  	if(this.getDimensiones()>=2){
	  	  	this.setX(coordenadas[0]);
	  	  	this.setY(coordenadas[1]);
  	  	}
    }
  
  
    public void setCluster(Cluster cluster){
    	this.cluster=cluster;
    }
  
    public long getClusterId(){
    	return cluster.ID;
    }
  
    public int getWho(){ return who; }
  
    public String toString(){ return "Agente"+who+"("+getX()+", "+getY()+", "+calculaVisibilidad()+")";}
  
    public Vector getCoordenadas(){
    	return new Vector(coordenadas);
    }
  
  public int getDimensiones(){
	  return coordenadas.length;
  }

}
